 /*
 *  File:       SxNano_Monitor_V33.ino
 *  Autor:      Frank Keil
 *  Controller: Arduino Nano (5V, 16MHz), ATmega328
 *  Hardware:   Sx-Decoder von Frank Keil 
 *              http://frank-keil.de/Selectrix/Sx-Decoder/Sx-Decoder.html
 *  
 *  Libraries:  SXArduino: Quellen von G.v.d.Sel  
 *              https://github.com/gvandersel/SXArduino
 *
 *  Created on: 02.01.2012
 *  Last changed on: 13.06.2021
 *  
 *  Beschreibung:
 *  Auf einem zweizeilen LCD-Display werden zwei Sx-Adressen angezeigt.
 *
 *  Pin2=T0=INT0, Pin4=T1, Pin3=SX-Write-High, Pin6=SX-Write-Low
 *
 *  Zum Programmieren des Moduls am Sx-Bus können die SX-Kanäle 0 bis 5 verwendet werden:
 *   Sx-Kanal 0 : Sx-Adresse 1 dieses Moduls (0..99)
 *   Sx-Kanal 1 : Sx-Adresse 2 dieses Moduls (0..99)
 *   Sx-Kanal 2 : -
 *   Sx-Kanal 3 : -
 *   Sx-Kanal 4 : -
 *   Sx-Kanal 5 : -
 *  Die alten Daten auf dem Bus werden gerettet und zurück geschrieben.
 *   
 *  Bit_16 ist während der Programmierung an und Messung an
 *
 *  SXbus.write(adr, val);      to set a value on an address of the SX-bus.
 *  val = SXbus.read(adr);      to get a value from the SXbus.
 *  SXbus.writePWR(val);        val = 0 for off, val = 1 for on (all other values have no influence)
 *  val = SXbus.readPWR();      val = 0 for off, val = 1 for on.

  */

  #include <LiquidCrystal.h>                  // Bibliothek LiquidCrystal einbinden
  LiquidCrystal lcd(A0, A1, A2, A3, A4, A5);  // initialisieren Bibliothek und Pins festlegen

  #include "SXArduino.h"                      // Lib einbinden
  // SXArduino SXbus(2,4,3,6);                // Get access to the SX-bus , OpenSx
  SXArduino SXbus(2,4,6,3);                   // Get access to the SX-bus , Sx-Nano FK
  bool noCC2000 = false;                      // false 0 ... 103, true 0 ... 111

  #include <EEPROM.h>                         // EEPROM Bibliothek hinzufügen

  int analogPin = 6;                          // Progtaster
  #define Bit_1 1                             // Led 1
  #define Bit_2 0                             // Led 2
  #define Bit_3 5                             // Led 3
  #define Bit_4 7                             // Led 4
  #define Bit_5 8                             // Led 5
  #define Bit_6 9                             // Led 6
  #define Bit_7 10                            // Led 7
  #define Bit_8 11                            // Led 8
  #define Bit_15 12                           // Led 15 Led - Gleis ein
  #define Bit_16 13                           // Led 16 Led - Prog aktiv
  byte DecAdr=10;                             // Decoder Adresse
  byte SxAdr1=10;                             // Sx Adresse 1
  byte SxData1=0;                             // aktuelle SxDaten 1                  
  byte OldSxData1=6;                          // altes Byte 1
  byte SxAdr2=10;                             // Sx Adresse 2
  byte SxData2=0;                             // aktuelle SxDaten 2                  
  byte OldSxData2=6;                          // altes Byte 2
  byte SxAdrSchreib=0;                        // SxAdressen die geschrieben werden 
  byte SxDataSchreib=0;                       // SxDaten die geschrieben werden  
  byte tb;                                    // Gleisbit
  byte oldtb;                                 // alter Wert Gleisbit
  int  progAkt;                               // Programmierung aktiv
  int  Prog = 0;                              // Analogwert Progtaster
  byte Vergleich=0;                           // Überprüft ob schon eine Prog-Anforderung anliegt

void setup()
  {
  SxAdr1 = EEPROM.read(1);                    // EEprom Zelle 1 auslesen
  SxAdr2 = EEPROM.read(2);                    // EEprom Zelle 2 auslesen
  lcd.begin(16,2);                            // LCD konfigurieren
  lcd.setCursor(1, 0);                        // Cursor an Pos setzen
  lcd.print("Sx-Busanzeige");                 // schreiben
  lcd.setCursor(1, 1);                        // Cursor an Pos setzen
  lcd.print("von Frank Keil");                // Ausgabe
  Anzeige();
  SXbus.init();                               // SX-bus Kommunikation
  attachInterrupt(0, sxisr, RISING);          // wechsel INT0 triggers the interrupt routine sxisr (see above)
  }

void sxisr(void) 
  { 
  SXbus.isr();
  } 

void loop()
  {
  tb = SXbus.read(109);                       // Info, ob das Gleissignal an/aus ist
    tb = bitRead(tb,7);
    if (oldtb != tb)                          // Gleisbit geändert?
        {
        oldtb = tb;                           // Gleisbit gleich setzen
        digitalWrite(Bit_15,tb);              // Led je nach Zustand ein/ausschalten
        // Serial.println(oldtb);             // Ausgabe
        }
        
  SxData1=SXbus.read(SxAdr1);                 // überprüfen ob sich etwas geändert hat
  if (OldSxData1 != SxData1)                  // Daten haben sich auf dem Bus geändert ?
      {
      OldSxData1 = SxData1;                   // aktueller Wert wird zu altem Wert 
      lcd.setCursor(13, 0);                   // Cursor an Pos setzen
      if(SxData1 <10) lcd.print(" ");         // ein leerzeichen  oder auch Zahl Null
      if(SxData1 <100) lcd.print(" ");        // ein leerzeichen
      lcd.print(SxData1);                     // schreiben
      lcd.setCursor(4, 0);                    // Cursor an Pos setzen
      if (SxData1 < 2) lcd.print("0");        // schreiben
      if (SxData1 < 4) lcd.print("0");        // schreiben
      if (SxData1 < 8) lcd.print("0");        // schreiben
      if (SxData1 < 16) lcd.print("0");       // schreiben
      if (SxData1 < 32) lcd.print("0");       // schreiben
      if (SxData1 < 64) lcd.print("0");       // schreiben
      if (SxData1 < 128) lcd.print("0");      // schreiben
      lcd.print(SxData1, BIN);                // schreiben
      delay(100);                             // warte
      }
  SxData2=SXbus.read(SxAdr2);                 // überprüfen ob sich etwas geändert hat
  if (OldSxData2 != SxData2)                  // Daten haben sich auf dem Bus geändert ?
      {
      OldSxData2 = SxData2;                   // aktueller Wert wird zu altem Wert 
      lcd.setCursor(13, 1);                   // Cursor an Pos setzen
      if(SxData2 <10) lcd.print(" ");         // ein leerzeichen  oder auch Zahl Null
      if(SxData2 <100) lcd.print(" ");        // ein leerzeichen
      lcd.print(SxData2);                     // schreiben
      lcd.setCursor(4, 1);                    // Cursor an Pos setzen
      if (SxData2 < 2) lcd.print("0");        // schreiben
      if (SxData2 < 4) lcd.print("0");        // schreiben
      if (SxData2 < 8) lcd.print("0");        // schreiben
      if (SxData2 < 16) lcd.print("0");       // schreiben
      if (SxData2 < 32) lcd.print("0");       // schreiben
      if (SxData2 < 64) lcd.print("0");       // schreiben
      if (SxData2 < 128) lcd.print("0");      // schreiben
      lcd.print(SxData2, BIN);                // schreiben
      }
  
  // ************************************************* Programmierroutine       
    if (progAkt == 0)                                 // wenn progAktiv nicht gesetzt
        {
        tb = SXbus.readPWR();                         // Info, ob das Gleissignal an/aus ist
        Prog = analogRead(analogPin);                 // read the input pin 
        if (Prog <= 500 && tb ==false)                // falls gedrückt und Gleis aus
            {
            SxData1 = SXbus.read(106);                // Sx-Adr 106 auslesen
            Vergleich = SxData1 & B00100000;          // Prog schon von anderem Decoder gesetzt?
            if (Vergleich != 32)                      // wenn Prog Anforderung auf Bus noch nicht aktiv ist
                {
                progAkt = 1;                          // setze progAktiv
                lcd.clear();                          // lösche Display
                lcd.setCursor(3,0);                   // Cursor positionieren
                lcd.print("Prog An");                 // Ausgabe
                digitalWrite(Bit_16,HIGH);            // Prog-Led einschalten
                SxDataSchreib = SxData1 | B00100000;  // Prog-Bit auf Sx-Bus setzen
                SxAdrSchreib = 106;                   // Sx Adresse die beschrieben werden soll
                Schreiben();                          // ins Unterprog springen
                delay (500);                          // warte
                SxData1=SXbus.read(0);                // Sx-Adr 00 auslesen
                EEPROM.write(10,SxData1);             // und in eEprom retten           
                SxData1=SXbus.read(1);                // Sx-Adr 01 auslesen
                EEPROM.write(11,SxData1);             // und in eEprom retten           
                SxData1=SXbus.read(2);                // Sx-Adr 02 auslesen
                EEPROM.write(12,SxData1);             // und in eEprom retten           
                
                SxAdr1 = EEPROM.read(1);              // EEprom Zelle 1 auslesen
                SxAdrSchreib=0;                       // Sx Adresse die beschrieben werden soll
                SxDataSchreib=SxAdr1;                 // setze Wert der geschrieben werden soll
                Schreiben();                          // ins Unterprog springen
                SxAdr2 = EEPROM.read(2);              // EEprom Zelle 2 auslesen
                SxAdrSchreib=1;                       // setze Adresse die geschrieben werden soll
                SxDataSchreib=SxAdr2;                 // setze Wert der geschrieben werden soll
                Schreiben();                          // ins Unterprog springen                                 
                }            
            else
                {
                lcd.clear();                          // lösche Display
                lcd.setCursor(0,0);                   // Cursor positionieren
                lcd.print("Prog Anfordung");          // Ausgabe
                lcd.setCursor(0,1);                   // Cursor positionieren
                lcd.print("bereits aktiv");           // Ausgabe
                }
            delay (1500);                             // warte
            }
        } 
    // ************************************************* Programmierroutine schreiben
    if (progAkt == 1)                                 // wenn progAktiv gesetzt
        {
        tb = SXbus.readPWR();                         // Info, ob das Gleissignal an/aus ist              
        Prog = analogRead(analogPin);                 // read the input pin 
        if (Prog <= 500 || tb ==true)                 // falls gedrückt oder Gleis ein
            {
            SxAdr1=SXbus.read(0);                     // Sx-Adr 00 auslesen
            EEPROM.write(1,SxAdr1);                   // in eEprom schreiben            
            SxAdr2 = SXbus.read(1);                   // Sx-Adr 01 auslesen
            EEPROM.write(2,SxAdr2);                   // in eEprom schreiben
            
            SxDataSchreib = EEPROM.read(10);          // EEprom Zelle 10 mit geretteten auslesen
            SxAdrSchreib=0;                           // Sx Adresse die beschrieben werden soll
            Schreiben();                              // ins Unterprog springen           
            SxDataSchreib = EEPROM.read(11);          // EEprom Zelle 11 mit geretteten auslesen
            SxAdrSchreib=1;                           // Sx Adresse die beschrieben werden soll
            Schreiben();                              // ins Unterprog springen           
            
            SxData1=SXbus.read(106);                  // Sx-Adr 106 auslesen
            SxDataSchreib=SxData1 & B11011111;        // Prog-Bit zurück setzen
            SxAdrSchreib=106;                         // Sx Adresse die beschrieben werden soll
            Schreiben();                              // ins Unterprog springen
            delay(1000);                              // warte
            progAkt = 0;                              // lösche progAktiv
            // Serial.println("Prog aus");            // Ausgabe
            lcd.clear();                              // lösche Display
            lcd.setCursor(3,0);                       // Cursor positionieren
            lcd.print("Prog Aus");                    // Ausgabe
            digitalWrite(Bit_16,LOW);                 // Led ausschalten
            Anzeige();                                // springe zum Unterprogramm
            }
        }
  }

void Schreiben()
    {    
    SXbus.write(SxAdrSchreib,SxDataSchreib);          // Wert auf Sx-Bus schreiben
    } 

void Anzeige()
  {
  delay(4000);                                // warte   
  lcd.clear();                                // lösche Display
  lcd.setCursor(0,0);                         // Cursor erste Reihe
  lcd.setCursor(0, 0);                        // Cursor an Pos setzen
  if(SxAdr1 <10)                              // wenn kleiner 10
      {
      lcd.print(" ");                         // ein leerzeichen  oder auch Zahl Null
      }
   if(SxAdr1 <100) lcd.print(" ");            // ein leerzeichen
   lcd.print(SxAdr1);                         // schreiben
   lcd.setCursor(0, 1);                       // Cursor an Pos setzen
   if(SxAdr2 <10)                             // wenn kleiner 10
      {
      lcd.print(" ");                         // ein leerzeichen  oder auch Zahl Null
      }
  if(SxAdr2 <100) lcd.print(" ");             // ein leerzeichen
  lcd.print(SxAdr2);                          // schreiben
  } 
